module bus_protocol(clk,a_0x,a_1x,a_2x,a_3x,a_4x,a_5x,a_6x,out_7x,out_8x,out_9x);
   input         clk;
   input 	 a_0x;	// external name: input`mw
   input 	 a_1x;	// external name: input`mr
   input 	 a_2x;	// external name: input`req
   input 	 a_3x;	// external name: input`burst
   input [7:0] 	 a_4x;	// external name: input`mwb
   input [28:0]  a_5x;	// external name: input`adr
   input [63:0]  a_6x;	// external name: input`din
   output 	 out_7x;	// external name: out``bus_output`reqp
   output 	 out_8x;	// external name: out``bus_output`brdy
   output [63:0] out_9x;	// external name: out``bus_output`dout

   // wire declarations
   reg [63:0] 	 data_in_capim;
   wire [63:0] 	 data_out_capim;
   reg 		 write_capim;
   reg [7:0] 	 BE_capim;
   reg 		 access_capim;
   reg [21:0] 	 addr_capim;
   wire 	 intr_capim;
   reg 		 inta_capim;

   wire [15:0] 	 sd_data0;
   wire [15:0] 	 sd_data1;
   wire [15:0] 	 sd_data2;
   wire [15:0] 	 sd_data3;
   wire [1:0] 	 sd_ba;
   wire [11:0] 	 sd_addx;
   wire 	 sd_ras_l;
   wire 	 sd_cas_l;
   wire 	 sd_wr_l;
   wire 	 sd_cs_l;
   wire [1:0] 	 sd_dqm0;
   wire [1:0] 	 sd_dqm1;
   wire [1:0] 	 sd_dqm2;
   wire [1:0] 	 sd_dqm3;

   reg 		 reset;
   
   // instantiations
   memory_unit mu1(
		   .clk(clk),
		   .reset(reset),
		   
		   // connections to cpu
		   .MAd_cpu({3'b000,a_5x}),
		   .MDat_in_cpu(a_6x),
		   .MDat_out_cpu(out_9x),
		   .req_cpu(a_2x),
		   .reqp_cpu(out_7x),
		   .BRdy_cpu(out_8x),
		   .wr_cpu(a_0x),
		   .BE_cpu(a_4x),
		   
		   // connections to capims
		   .data_in_capim(data_in_capim),
		   .data_out_capim(data_out_capim),
		   .write_capim(write_capim),
		   .BE_capim(BE_capim),
		   .access_capim(access_capim),
		   .addr_capim(addr_capim),
		   .intr_capim(intr_capim),
		   .inta_capim(inta_capim),
		   
		   // connection to sdram chips
		   .sd_data0(sd_data0),
		   .sd_data1(sd_data1),
		   .sd_data2(sd_data2),
		   .sd_data3(sd_data3),
		   .sd_ba(sd_ba),
		   .sd_addx(sd_addx),
		   .sd_ras_l(sd_ras_l),
		   .sd_cas_l(sd_cas_l),
		   .sd_wr_l(sd_wr_l),
		   .sd_cs_l(sd_cs_l),
		   .sd_dqm0(sd_dqm0),
		   .sd_dqm1(sd_dqm1),
		   .sd_dqm2(sd_dqm2),
		   .sd_dqm3(sd_dqm3));

   // Instantiierung der SDRAM-Chips
   SDRAM64Mx16 SDRAMchip0 (.DQ15(sd_data0[15]),
                           .DQ14(sd_data0[14]),
                           .DQ13(sd_data0[13]),
                           .DQ12(sd_data0[12]),
                           .DQ11(sd_data0[11]),
                           .DQ10(sd_data0[10]),
                           .DQ9(sd_data0[9]),
                           .DQ8(sd_data0[8]),
                           .DQ7(sd_data0[7]),
                           .DQ6(sd_data0[6]),
                           .DQ5(sd_data0[5]),
                           .DQ4(sd_data0[4]),
                           .DQ3(sd_data0[3]),
                           .DQ2(sd_data0[2]),
                           .DQ1(sd_data0[1]),
                           .DQ0(sd_data0[0]),
                           .A13(sd_ba[1]),
                           .A12(sd_ba[0]),
                           .A11(sd_addx[11]),
                           .A10(sd_addx[10]),
                           .A9(sd_addx[9]),
                           .A8(sd_addx[8]),
                           .A7(sd_addx[7]),
                           .A6(sd_addx[6]),
                           .A5(sd_addx[5]),
                           .A4(sd_addx[4]),
                           .A3(sd_addx[3]),
                           .A2(sd_addx[2]),
                           .A1(sd_addx[1]),
                           .A0(sd_addx[0]),
                           .CLK(clk),
                           .CKE(1'b1),
                           .RAS(sd_ras_l),
                           .CAS(sd_cas_l),
                           .WE(sd_wr_l),
                           .CS(sd_cs_l),
                           .DQMU(sd_dqm0[1]),
                           .DQML(sd_dqm0[0]));

   SDRAM64Mx16 SDRAMchip1 (.DQ15(sd_data1[15]),
                           .DQ14(sd_data1[14]),
                           .DQ13(sd_data1[13]),
                           .DQ12(sd_data1[12]),
                           .DQ11(sd_data1[11]),
                           .DQ10(sd_data1[10]),
                           .DQ9(sd_data1[9]),
                           .DQ8(sd_data1[8]),
                           .DQ7(sd_data1[7]),
                           .DQ6(sd_data1[6]),
                           .DQ5(sd_data1[5]),
                           .DQ4(sd_data1[4]),
                           .DQ3(sd_data1[3]),
                           .DQ2(sd_data1[2]),
                           .DQ1(sd_data1[1]),
                           .DQ0(sd_data1[0]),
                           .A13(sd_ba[1]),
                           .A12(sd_ba[0]),
                           .A11(sd_addx[11]),
                           .A10(sd_addx[10]),
                           .A9(sd_addx[9]),
                           .A8(sd_addx[8]),
                           .A7(sd_addx[7]),
                           .A6(sd_addx[6]),
                           .A5(sd_addx[5]),
                           .A4(sd_addx[4]),
                           .A3(sd_addx[3]),
                           .A2(sd_addx[2]),
                           .A1(sd_addx[1]),
                           .A0(sd_addx[0]),
                           .CLK(clk),
                           .CKE(1'b1),
                           .RAS(sd_ras_l),
                           .CAS(sd_cas_l),
                           .WE(sd_wr_l),
                           .CS(sd_cs_l),
                           .DQMU(sd_dqm1[1]),
                           .DQML(sd_dqm1[0]));

   SDRAM64Mx16 SDRAMchip2 (.DQ15(sd_data2[15]),
                           .DQ14(sd_data2[14]),
                           .DQ13(sd_data2[13]),
                           .DQ12(sd_data2[12]),
                           .DQ11(sd_data2[11]),
                           .DQ10(sd_data2[10]),
                           .DQ9(sd_data2[9]),
                           .DQ8(sd_data2[8]),
                           .DQ7(sd_data2[7]),
                           .DQ6(sd_data2[6]),
                           .DQ5(sd_data2[5]),
                           .DQ4(sd_data2[4]),
                           .DQ3(sd_data2[3]),
                           .DQ2(sd_data2[2]),
                           .DQ1(sd_data2[1]),
                           .DQ0(sd_data2[0]),
                           .A13(sd_ba[1]),
                           .A12(sd_ba[0]),
                           .A11(sd_addx[11]),
                           .A10(sd_addx[10]),
                           .A9(sd_addx[9]),
                           .A8(sd_addx[8]),
                           .A7(sd_addx[7]),
                           .A6(sd_addx[6]),
                           .A5(sd_addx[5]),
                           .A4(sd_addx[4]),
                           .A3(sd_addx[3]),
                           .A2(sd_addx[2]),
                           .A1(sd_addx[1]),
                           .A0(sd_addx[0]),
                           .CLK(clk),
                           .CKE(1'b1),
                           .RAS(sd_ras_l),
                           .CAS(sd_cas_l),
                           .WE(sd_wr_l),
                           .CS(sd_cs_l),
                           .DQMU(sd_dqm2[1]),
                           .DQML(sd_dqm2[0]));

   SDRAM64Mx16 SDRAMchip3 (.DQ15(sd_data3[15]),
                           .DQ14(sd_data3[14]),
                           .DQ13(sd_data3[13]),
                           .DQ12(sd_data3[12]),
                           .DQ11(sd_data3[11]),
                           .DQ10(sd_data3[10]),
                           .DQ9(sd_data3[9]),
                           .DQ8(sd_data3[8]),
                           .DQ7(sd_data3[7]),
                           .DQ6(sd_data3[6]),
                           .DQ5(sd_data3[5]),
                           .DQ4(sd_data3[4]),
                           .DQ3(sd_data3[3]),
                           .DQ2(sd_data3[2]),
                           .DQ1(sd_data3[1]),
                           .DQ0(sd_data3[0]),
                           .A13(sd_ba[1]),
                           .A12(sd_ba[0]),
                           .A11(sd_addx[11]),
                           .A10(sd_addx[10]),
                           .A9(sd_addx[9]),
                           .A8(sd_addx[8]),
                           .A7(sd_addx[7]),
                           .A6(sd_addx[6]),
                           .A5(sd_addx[5]),
                           .A4(sd_addx[4]),
                           .A3(sd_addx[3]),
                           .A2(sd_addx[2]),
                           .A1(sd_addx[1]),
                           .A0(sd_addx[0]),
                           .CLK(clk),
                           .CKE(1'b1),
                           .RAS(sd_ras_l),
                           .CAS(sd_cas_l),
                           .WE(sd_wr_l),
                           .CS(sd_cs_l),
                           .DQMU(sd_dqm3[1]),
                           .DQML(sd_dqm3[0]));
endmodule
