`timescale 100ps / 100ps

module dlxtom_simulator;
   // registers
   reg         clk;
   reg 	       reset;
   wire [28:0] bus_protocol_address;  
   wire [63:0] bus_protocol_din;  
   wire        bus_protocol_req;   
   wire        bus_protocol_wr;   
   wire        bus_protocol_burst;   
   wire [7:0]  bus_protocol_be;   
   wire [63:0] bus_protocol_dout;   
   wire        bus_protocol_reqp;   
   wire        bus_protocol_brdy;
   reg [28:0]  ext_input_address;   
   reg [63:0]  ext_input_dout;   
   reg 	       ext_input_mw;   
   reg 	       ext_input_mr;   
   reg [7:0]   ext_input_mbw;   
   reg 	       ext_input_cache_reset;   
   wire [63:0] ext_output_din;
   wire        ext_output_busy;
   
   wire [21:0] sdc_address;
   wire [63:0] sdc_in_data;
   reg [63:0]  sdc_out_data;
   wire        sdc_rnw;
   wire        sdc_req;
   reg 	       sdc_ack;   
   reg 	       sdc_out_data_valid;

   wire        gnulp_ready;
   
   
   // instantiation of dlxtom
   tom_main tom1(.clk(clk),
		 .reset(reset),
		 .bus_protocol_address(bus_protocol_address),
		 .bus_protocol_din(bus_protocol_din),
		 .bus_protocol_req(bus_protocol_req),
		 .bus_protocol_wr(bus_protocol_wr),
		 .bus_protocol_burst(bus_protocol_burst),
		 .bus_protocol_be(bus_protocol_be),
		 .bus_protocol_dout(bus_protocol_dout),
		 .bus_protocol_reqp(bus_protocol_reqp),
		 .bus_protocol_brdy(gnulp_ready),
		 .ext_input_address(ext_input_address),
		 .ext_input_dout(ext_input_dout),
		 .ext_input_mw(ext_input_mw),
		 .ext_input_mr(ext_input_mr),
		 .ext_input_mbw(ext_input_mbw),
		 .ext_input_cache_reset(ext_input_cache_reset),
		 .ext_output_din(ext_output_din),
		 .ext_output_busy(ext_output_busy));
   
   memory_unit mu1(.clk(clk),
		   .reset(ext_input_cache_reset), 
		   
		   // connections to cpu
		   .MAd_cpu(bus_protocol_address),
		   .MDat_in_cpu(bus_protocol_din),
		   .MDat_out_cpu(bus_protocol_dout),
		   .req_cpu(bus_protocol_req),
		   .reqp_cpu(bus_protocol_reqp),
		   .BRdy_cpu(gnulp_ready),
		   .wr_cpu(bus_protocol_wr),
		   .burst(bus_protocol_burst),
		   
		   // connection to Isytec-Memory-Controller
		   .sdc_address(sdc_address),
		   .sdc_in_data(sdc_in_data),
		   .sdc_out_data(sdc_out_data),
		   .sdc_rnw(sdc_rnw),
		   .sdc_req(sdc_req),
		   .sdc_ack(sdc_ack),
		   .sdc_out_data_valid(sdc_out_data_valid));
      
   // simulation stimuli
   always
     #500 clk=~clk;



   initial
     begin

	$shm_open("dlxtom2.shm");
	$shm_probe("A",tom1,"A",tom1.tom1,"A",mu1,"A",mu1.mp_mem1,"A",mu1.arbiter1,"A");

	ext_input_mw=0;
	ext_input_mr=0;
	ext_input_mbw=8'b11111111;
	sdc_out_data_valid=0;
	
	sdc_ack=0;	
	
	reset=1;     
	clk=1;
	ext_input_cache_reset=1;      

	// Cache starten
	#1100 ext_input_cache_reset=0;
	
	// Programm in den Speicher schreiben
	#3000 ext_input_address=0;
	ext_input_mw=1;
	ext_input_dout=64'h240201002401000a;

	@sdc_req #2000;
	sdc_ack=1;
	#4000 sdc_ack=0;       
	#2000 sdc_out_data_valid =1;
	sdc_out_data=55555;	
	#4000 sdc_out_data_valid=0;
	sdc_out_data=64'bx;       
	
	@ext_output_busy #1000;
	ext_input_mw=0;
	
	#3000 ext_input_address=2;
	ext_input_mw=1;
	ext_input_dout=64'he44400004404080a;
	@ext_output_busy #1000;
	ext_input_mw=0;
	
	
	#3000 ext_input_address=3;
	ext_input_mw=1;
	ext_input_dout=64'hac43000844830009;
	@ext_output_busy #1000;
	ext_input_mw=0;

	#3000 ext_input_address=4;
	ext_input_mw=1;
	ext_input_dout=64'h240000001000fffc;
		
	
	@sdc_req #2000;
	sdc_ack=1;
	#4000 sdc_ack=0;       
	#2000 sdc_out_data_valid =1;
	sdc_out_data=66666;	
	#4000 sdc_out_data_valid=0;
	sdc_out_data=64'bx;
	
	@ext_output_busy #1000;
	ext_input_mw=0;
	
	#3000 ext_input_address=1;
	ext_input_mw=1;
	ext_input_dout=64'h2400000024000000;
	
	@ext_output_busy #1000;
	ext_input_mw=0;

	

       
	
	
	// Vamp starten
	#5000 reset=0;

	
	@sdc_req #5000;
	sdc_ack=1;
	#4000 sdc_ack=0;
	
	
 //      	@bus_protocol_req #1000;
//	bus_protocol_reqp=1;
//	bus_protocol_brdy=1;
//	#3000 bus_protocol_reqp=0;
//	#1000 bus_protocol_brdy=0;

	
//	@bus_protocol_req #1000;
//	bus_protocol_reqp=1;
//	bus_protocol_brdy=1;
//	#3000 bus_protocol_reqp=0;
//	#1000 bus_protocol_brdy=0;
	
#100000
  $finish;
	
//	@bus_protocol_req #1000;
//	bus_protocol_reqp=1;
//	bus_protocol_brdy=1;
//	#3000 bus_protocol_reqp=0;
//	#1000 bus_protocol_brdy=0;

	
	#100000;
	$finish;   	
     end
endmodule
