module memory_unit(
		   clk,
		   reset, 
		   
		   // connections to cpu
		   MAd_cpu,
		   MDat_in_cpu,
		   MDat_out_cpu,
		   req_cpu,
		   reqp_cpu,
		   BRdy_cpu,
		   wr_cpu,
		   burst,
		   
		   // connection to Isytec-Memory-Controller
		   sdc_address,
		   sdc_in_data,
		   sdc_out_data,
		   sdc_rnw,
		   sdc_req,
		   sdc_ack,
		   sdc_out_data_valid);
   

   input		clk;
   input	        reset;
   // connections to cpu
   input [28:0] MAd_cpu;
   input [63:0] MDat_in_cpu;
   output [63:0] MDat_out_cpu;
   input 	 req_cpu;
   output 	 reqp_cpu;
   output 	 BRdy_cpu;
   input 	 wr_cpu;
   input 	 burst;      

   // connection to isytec SDRAM-Controller
   output [21:0] sdc_address;
   output [63:0] sdc_in_data;
   input [63:0]  sdc_out_data;
   output 	 sdc_rnw;
   output 	 sdc_req;
   input 	 sdc_ack;
   input 	 sdc_out_data_valid;
   
   // connections between arbiter and mp_mem_interface
   wire [31:0] 	MAd_arb;
   wire [63:0] 	MDat_in_arb;
   wire [63:0] 	MDat_out_arb;
   wire		req_arb;
   wire 	reqp_arb;
   wire 	BRdy_arb;
   wire 	wr_arb;
   
 
   // instantiation of arbiter
   memory_arbiter arbiter1(
			   .clk(clk),
			   .reset(reset),
			   .MAd_arb(MAd_arb),
			   .MDat_in_arb(MDat_in_arb),
			   .MDat_out_arb(MDat_out_arb),
			   .burst_cpu(burst),
			   .req_arb(req_arb),
			   .reqp_arb(reqp_arb),
			   .BRdy_arb(BRdy_arb),
			   .wr_arb(wr_arb),
			   .MAd_cpu({3'b000,MAd_cpu}),
			   .MDat_in_cpu(MDat_in_cpu),
			   .MDat_out_cpu(MDat_out_cpu),
			   .req_cpu(req_cpu),
			   .reqp_cpu(reqp_cpu),
			   .BRdy_cpu(BRdy_cpu),
			   .wr_cpu(wr_cpu),
			   .data_in_capim(64'b0),			   
			   .addr_capim(22'b0),
			   .write_capim(1'b0),			   
			   .access_capim(1'b0),
			   .inta_capim(1'b0));
   
   // Instantiation of mp_memory_interface
   mp_mem_interface_64 mp_mem1 ( .sys_rst_l(!reset),
				 .sys_clk(clk),
				 .sdc_address(sdc_address),
				 .sdc_in_data(sdc_in_data),
				 .sdc_out_data(sdc_out_data),
				 .sdc_rnw(sdc_rnw),
				 .sdc_req(sdc_req),
				 .sdc_ack(sdc_ack),
				 .sdc_out_data_valid(sdc_out_data_valid),
				 .MAd(MAd_arb),
				 .MDat_in(MDat_in_arb),
				 .MDat_out(MDat_out_arb),
				 .req(req_arb),
				 .reqp(reqp_arb),
				 .BRdy(BRdy_arb),
				 .wr(wr_arb),
//				 .BE(BE_arb),
				 .burst(burst)
				 );
endmodule